#ifndef __change_h_
#define __change_h_

#include	"mesh.h"
#include	<vector>

namespace	VMI
{
	typedef struct change
	{
		int e;
		int u, v;           // Edge
		int numDel;         // Number of deleted triangles
		Triangle *deleted;  // List of deleted triangles 
		int numMod;         // Number of modified triangles 
		Triangle *modified; // List of triangles deleted  
	} Change;

	// Represents a simplification step in the sequence.
	struct vmiStep
	{
		unsigned int	mV0;
		unsigned int	mV1;
		unsigned int	mT0;
		unsigned int	mT1;
		float x;
		float	y;
		float	z;
		
		std::vector<unsigned int>	mModfaces;

		//	Indicates the the step is obligatory to execute
		//	joined width the following step.
		unsigned int obligatory;
	};

	// Stores all the simplification steps.
	extern	std::vector<vmiStep> mVMISteps;

	extern Change *createChange (Mesh *mesh, int e);
	extern void writeChange(FILE* file, Change *c);
	extern void deleteChange(Change *c);
	extern void printChange(Change *c);

	extern void modifyTriangle(Triangle *t, int c, int p);
	extern int isATriangleToModify(Triangle *t, int c, int p);
	extern int getTrianglesToModify(Mesh *mesh, int c, int p, Triangle *modified);
	extern int isATriangleToDelete(Triangle *t, int c, int p);
	extern int getTrianglesToDelete(Mesh *mesh, int numMod, Triangle *modified, Triangle *deleted, int c, int p);

	extern void modifyTriangles(Mesh *mesh, Change *c);
	extern void unmodifyTriangles(Mesh *mesh, Change *c);
	extern void deleteTriangles(Mesh *mesh, Change *c);
	extern void undeleteTriangles(Mesh *mesh, Change *c);

	extern void printList(Triangle *list, int n);
	extern void deleteItem(Triangle *list, int *n, int item);

	extern void doChange(Mesh *mesh, Change *c);
	extern void undoChange(Mesh *mesh, Change *c);
	extern void computeChanges(Mesh *mesh, Change *c);

	//	Save simplification sequence in Geometry Game Tools format.
	extern void	saveSimplificationSequence(Change	*c);
}

#endif
